// TLock structure contains time for ON and OFF state of the relay,
//   as well as flag which notifies if changes are made

struct TLock{
  char on_seconds, on_minutes, on_hours;
  char off_seconds, off_minutes, off_hours;
  char modified;
} Locks[4];  // we will use 4 relays/locks

char seconds, old_seconds, minutes, hours, day, month, year;

// LCD module connections
sbit  LCD_RS at LATB2_bit;
sbit  LCD_EN at LATB3_bit;
sbit  LCD_D4 at LATB4_bit;
sbit  LCD_D5 at LATB5_bit;
sbit  LCD_D6 at LATB6_bit;
sbit  LCD_D7 at LATB7_bit;
sbit  LCD_RS_Direction at TRISB2_bit;
sbit  LCD_EN_Direction at TRISB3_bit;
sbit  LCD_D4_Direction at TRISB4_bit;
sbit  LCD_D5_Direction at TRISB5_bit;
sbit  LCD_D6_Direction at TRISB6_bit;
sbit  LCD_D7_Direction at TRISB7_bit;
// End LCD module connections

// Keypad module connections
char keypadPort at PORTD;
// End Keypad module connections

// Software I2C connections
sbit Soft_I2C_Scl at RC3_bit;
sbit Soft_I2C_Sda  at RC4_bit;
sbit Soft_I2C_Scl_Direction at TRISC3_bit;
sbit Soft_I2C_Sda_Direction at TRISC4_bit;
// End Software I2C connections

// few auxiliary variables
char kp, ShowTime, SetupTime, noChange;
char index, CurrentLock;

// Performs project-wide initialization
void Init_All(){

  ANSELA = 0x00;
  ANSELB = 0x00;
  ANSELC = 0x00;
  ANSELD = 0x00;
  ANSELE = 0x00;
  SLRCON = 0;
  
  PORTA = 0x00;              // PORTA is output (relays)
  TRISA = 0x00;              // Initially all relays are off

  Soft_I2C_Init();            // Initialize Soft I2C communication

  Lcd_Init();                 // Initialize LCD
  Lcd_Cmd(_LCD_CLEAR);        // Clear LCD display
  
  Keypad_Init();              // Initialize Keypad

  kp = 0;                    // Set globals to start values
  index = 0;
  ShowTime = 0;
  SetupTime = 0;
  noChange = 0xFF;
  
  for (CurrentLock = 0; CurrentLock <=3; CurrentLock++){     // initialize times to some meaningfull values
     Locks[CurrentLock].on_hours    = 10;
     Locks[CurrentLock].on_minutes  = 37;
     Locks[CurrentLock].on_seconds  = 1  + CurrentLock;
     Locks[CurrentLock].off_hours   = 10;
     Locks[CurrentLock].off_minutes = 37;
     Locks[CurrentLock].off_seconds = 30 + CurrentLock;
     Locks[CurrentLock].modified    = 0xFF;
   }

  CurrentLock = 0;           // Position to the first lock
}