// Real Time Clock routines
// Format date and time
#include "unglobals.h"

static void Transform_Time(){
  seconds =  ((seconds & 0xF0) >> 4)*10 + (seconds & 0x0F);  // Transform seconds
  minutes =  ((minutes & 0xF0) >> 4)*10 + (minutes & 0x0F);  // Transform months
  hours   =  ((hours   & 0xF0) >> 4)*10 + (hours & 0x0F);    // Transform hours
}

// Read time and date information from RTC DS1307
void Read_Time(){
  if (SetupTime) {  // if we are in setup mode then do not read from RTC
    return;
  }
    
  Soft_I2C_Start();                // Issue start signal
  Soft_I2C_Write(0xA0);            // Address PCF8583, see PCF8583 datasheet
  Soft_I2C_Write(2);               // Start from address 2
  Soft_I2C_Start();                // Issue repeated start signal
  Soft_I2C_Write(0xA1);            // Address PCF8583 for reading R/W=1
  seconds  = Soft_I2C_Read(1);     // Read seconds byte
  minutes  = Soft_I2C_Read(1);     // Read minutes byte
  hours    = Soft_I2C_Read(1);     // Read hours byte
  day      = Soft_I2C_Read(1);     // Read year/day byte
  month    = Soft_I2C_Read(0);     // Read weekday/month byte

  Soft_I2C_Stop();                 // Issue stop signal

  Transform_Time();                // Format date and time
}

void Set_Time(char hour, char minute){
char tmp1, tmp2;

   tmp1 = hour / 10;
   tmp2 = hour % 10;
   hour = tmp1 * 16 + tmp2;

   tmp1 = minute / 10;
   tmp2 = minute % 10;
   minute = tmp1 * 16 + tmp2;

   Soft_I2C_Start();             // issue start signal
   Soft_I2C_Write(0xA0);         // address PCF8583
   Soft_I2C_Write(0);            // start from word at address (REG0)
   Soft_I2C_Write(0x80);         // write $80 to REG0. (pause counter + 0 sec)
   Soft_I2C_Write(0);            // write 0 to seconds
   Soft_I2C_Write(0);            // write 0 to seconds
   Soft_I2C_Write(minute);       // write to minutes word to (REG1)
   Soft_I2C_Write(hour);         // write to hours word (24-hours mode)(REG2)
   Soft_I2C_Write(0x02);         // write 2 - Monday (REG3)
   Soft_I2C_Write(0x09);         // write 09 to year word (REG6)
   Soft_I2C_Stop();              // issue stop signal

   Soft_I2C_Start();             // issue start signal
   Soft_I2C_Write(0xA0);         // address PCF8583
   Soft_I2C_Write(0);            // start from word at address 0
   Soft_I2C_Write(0);            // write 0 to REG0 (enable counting + 0 sec)
   Soft_I2C_Stop();              // issue stop signal
}